//=============================================================================
// (c) Copyright 2009 Diamond Systems Corporation. Use of this source code
// is subject to the terms of Diamond Systems' Software License Agreement.
// Diamond Systems provides no warranty of proper performance if this
// source code is modified.
//
// File: DSCADScan.c   v1.00
//
// Depedency :	Universal Driver version 6.02
//				DSCUD.H - Include file.
//				DSCUDBCL.LIB - Library file for MS-DOS 
//						( USE BC 5.02 compiler )
//				LIBDSCUD_6.02_1.a	- Library file for Linux 2.6.23
//						( USE GCC 3.xx compiler )
//				DSCUD.LIB	- Static library file for Windows XP
//						( USE VC++ 6.0 or Visual Studio 2005 )
//				DSCUD_API.LIB	- Static library for Windows CE 6.0
//						( Use Visual Studio 2005 with Platform Builder 6.0 )
// Description: This sample code demonstrates the usage of UD function  
//				dscADScan(). This sample code describes the usage
//				for Helios SBC. Hardware on the board will be commanded
//              to scan the A/D channels passed, and return them to the
//              sample array. Then the values will be interpretted based on
//              the mode supplied by the user.
//              Each A/D hardware scan consists of a sweep of all the 
//              selected channels at a time, rather than only one channel.
// ************************* History ********************
// Version		Engineer		Date		Description		
// 1.0			James Moore		07/27/09	Created.
//=============================================================================

#include <stdio.h>

#ifdef _WIN32
#ifndef _WIN32_WCE
#include <conio.h>
#endif

#include <windows.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>
// diamond driver includes
#include "dscud.h"
#endif

// DOS
#ifdef __BORLANDC__
#include <conio.h>
#include <dos.h>
#include <stdlib.h>
#include <math.h>
#include <mem.h>
// diamond driver includes
#include "dscud.h"
#endif

// Linux
#if defined(linux)
#include <stdlib.h>
#include <math.h>
#include <time.h>
#include <string.h>
// diamond driver includes
#include "dscud.h"

#include <sys/time.h>

#endif

#define HELIOS_DEFAULT_BASE_ADDRESS 0x280

#define HELIOS_MAX_AD_CHANNEL_NUMBER 15

// var declarations
BYTE result; // returned error code
DSCB dscb;   // handle used to refer to the board
DSCCB dsccb; // structure containing board settings
DSCADSETTINGS dscadsettings; // structure containing A/D conversion settings
DSCADSCAN dscadscan; // structure containing A/D scan settings
DSCSAMPLE* samples;  // sample readings
DFLOAT voltage;      // actual voltage
ERRPARAMS errorParams;  // structure for returning error code and error string
int intBuff;            // temp variable of size int
int i;       // counter for the samples
int k;       // counter for the channels

char input_buffer[20]; // Buffer to keep user character input
int user_input_reset = 0;
char temp_char = 0;

//=============================================================================
// Name: main()
// Desc: Starting function that calls the driver functions used
//
//		 NOTE: By convention, you should capture the BYTE return value for each
//		 driver API call, and check the error code.
//
//     STEPS TO FOLLOW:
//
//	     I. Driver Initialization
//	    II. Board Initialization
//	   III. AD Settings Initialization
//		IV. AD Scan Initialization
//	     V. Scanning and Output
//	    VI. Cleanup
//
//=============================================================================

int main()
{

	//=========================================================================
	// I. DRIVER INITIALIZATION
	//
	//    Initializes the DSCUD library.
	//
	//    STEPS TO FOLLOW:
	//
	//	  1. initialize the driver, using the driver version for validation
	//=========================================================================

	if ( dscInit ( DSC_VERSION ) != DE_NONE )
	{
		dscGetLastError(&errorParams);
		fprintf ( stderr, "dscInit error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );
		return 0;
	}

	//=========================================================================
	// II. BOARD INITIALIZATION
	//
	//	   Initialize the HELIOS board. This function passes the various
	//	   hardware parameters to the driver and resets the hardware.
	//
	//     STEPS TO FOLLOW:
	//
	//	   1. set the board type to DSC_HELIOS for HELIOS board
	//	   2. set the base address to 0x280
	//	   3. set the interrupt level to 5
	//	   4. intialize and register the board with the driver, after which
	//		  the struct, dscb, now holds the handle for the board
	//=========================================================================

	printf ( "\nHELIOS BOARD INITIALIZATION:\n" );

	dsccb.boardtype = DSC_HELIOS;

	// This program uses command-line arguments as a convenience; they are not required.
	printf ( "Enter the base address (default 0x280, hit ENTER) : " );
	fgets ( input_buffer, 20, stdin );
	if ( input_buffer[0] == '\n' )
	{
	    dsccb.base_address = HELIOS_DEFAULT_BASE_ADDRESS;
	}
	else
	{
    	sscanf ( input_buffer, "%hx", &dsccb.base_address );
	}

    dsccb.int_level = 5;

	if ( dscInitBoard ( DSC_HELIOS, &dsccb, &dscb ) != DE_NONE )
	{
		dscGetLastError(&errorParams);
		fprintf ( stderr, "dscInitBoard error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );
		return 0;
	}

	//=========================================================================
	// III. AD SETTINGS INITIALIZATION
	//
	//	    Initialize the structure containing the AD conversion settings and
	//		then pass it to the driver.
	//
	//      STEPS TO FOLLOW:
	//
	//		1. set the polarity (must be BIPOLAR or UNIPOLAR)
	//		2. set the gain (must be GAIN_1, GAIN_2, GAIN_4, or GAIN_8)
	//		3. set the current channel to be sampled (must be between 0-15)
	//		4. initialize the AD conversion with these settings
	//=========================================================================

	printf ( "\nAD SETTINGS INITIALIZATION\n" );

	memset(&dscadsettings, 0, sizeof(DSCADSETTINGS));

	printf ( "Enter the polarity (0 for BIPOLAR, 1 for UNIPOLAR; default: 0): " );
	fgets ( input_buffer, 20, stdin );
	if ( input_buffer[0] == '\n' )
	{
    	dscadsettings.polarity = 0;
	}
	else
	{
        sscanf ( input_buffer, "%d", &intBuff );
    	dscadsettings.polarity = (BYTE) intBuff;
	}

	printf ( "Enter 0 for Single-Ended, 1 for Differential (default: 0): " );
	fgets ( input_buffer, 20, stdin );
	if ( input_buffer[0] == '\n' )
	{
    	dscadsettings.addiff = 0;
	}
	else
	{
        sscanf ( input_buffer, "%d", &intBuff );
    	dscadsettings.addiff = (BYTE) intBuff;
	}

	// The Helios only has one physical 10V range.
	dscadsettings.range = RANGE_10;

	printf ( "Enter the gain\n(0 for GAIN 1, 1 for GAIN 2, 2 for GAIN 4, 3 for GAIN 8; default: 0): " );
	fgets ( input_buffer, 20, stdin );
	if ( input_buffer[0] == '\n' )
	{
    	dscadsettings.gain = 0;
	}
	else
	{
        sscanf ( input_buffer, "%d", &intBuff );
    	dscadsettings.gain = (BYTE) intBuff;
	}

	// This feature will load the A/D calibration settings from the EEPROM

	printf ( "Load corresponding A/D Calibration settings? (0: no,1: yes; default: 0): " );
	fgets ( input_buffer, 20, stdin );
	if ( input_buffer[0] == '\n' )
	{
    	dscadsettings.load_cal = 0;
	}
	else
	{
        sscanf ( input_buffer, "%d", &intBuff );
    	dscadsettings.load_cal = (BYTE) intBuff;
	}

	dscadsettings.current_channel = 0;

	if ( ( result = dscADSetSettings ( dscb, &dscadsettings ) ) != DE_NONE )
	{
		dscGetLastError(&errorParams);
		fprintf ( stderr, "dscADSetSettings error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );
		return 0;
	}

	//=========================================================================
	// IV. AD SCAN INITIALIZATION
	//
	//	   Initialize the structure containing the AD scan setting and allocate
	//	   memory for our buffer containing sample values.
	//
	//     STEPS TO FOLLOW:
	//
	//	   1. set low channel (must be between 0-15)
	//	   2. set high channel (must be between 0-15)
	//	   3. set gain (must be GAIN_1, GAIN_2, GAIN_4, or GAIN_8; certain
	//		  SMM boards use GAIN_1, GAIN_10, GAIN_100, and GAIN_1000)
	//	   4. allocate memory for the sample values
	//=========================================================================

	printf ( "\nAD SCAN INITIALIZATION\n" );

    printf ( "Enter 0 for Single-Ended, 1 for Differential (default: 0): " );
	fgets ( input_buffer, 20, stdin );
	if ( input_buffer[0] == '\n' )
	{
		dscadsettings.addiff = 0;
	}
	else
	{
        sscanf ( input_buffer, "%d", &intBuff );
		dscadsettings.addiff = (BYTE) intBuff;
	}

    printf ( "(Single-Ended (0-%d)/Differential (0-%d))\n", HELIOS_MAX_AD_CHANNEL_NUMBER, HELIOS_MAX_AD_CHANNEL_NUMBER / 2 );	
    printf ( "Enter the lowest channel in the scan range (0-%d; default: 0): ", HELIOS_MAX_AD_CHANNEL_NUMBER );
	fgets ( input_buffer, 20, stdin );
	if ( input_buffer[0] == '\n' )
	{
    	dscadscan.low_channel = 0;
	}
	else
	{
        sscanf ( input_buffer, "%d", &intBuff );
	    dscadscan.low_channel = (BYTE) intBuff;
	}

	printf ( "Enter the highest channel in the scan range (0-%d; default: %d): ", HELIOS_MAX_AD_CHANNEL_NUMBER, HELIOS_MAX_AD_CHANNEL_NUMBER );
	fgets ( input_buffer, 20, stdin );
	if ( input_buffer[0] == '\n' )
	{
    	dscadscan.high_channel = HELIOS_MAX_AD_CHANNEL_NUMBER;
	}
	else
	{
        sscanf ( input_buffer, "%d", &intBuff );
	    dscadscan.high_channel = (BYTE) intBuff;
	}

	dscadscan.gain = dscadsettings.gain;

	samples = (DSCSAMPLE*)malloc( sizeof(DSCSAMPLE) * ( dscadscan.high_channel - dscadscan.low_channel + 1 ) );


	//=========================================================================
	// V. SCANNING AND OUTPUT
	//
	//    Perform the actual sampling and then output the results. To calculate
	//	  the actual input voltages, we must convert the sample code (which
	//	  must be cast to a short to get the correct code) and then plug it
	//	  into one of the formulas located in the manual for your board (under
	//	  "A/D Conversion Formulas"). For example, if you are using a bipolar
	//	  input range and 32-bit signed integers:
	//
	//	  Input voltage = (AD Code / 32768) x Full-Scale Voltage
	//
	//    STEPS TO FOLLOW:
	//
	//    1. get the samples from the board
	//    2. convert the results to a signed short to obtain a readout from
	//       -32768 to +32767 which scales to a range from -10V to +10V and
	//	     output the sample codes and the actual voltages to console
	//    3. calculate the input voltages by converting the sample code and
	//		 output the results
	//	  4. repeat steps 1-3 until a key is pressed
	//=========================================================================

	printf ( "\nSCANNING AND OUTPUT\n" );

	do 
	{

	    if ( temp_char == 'i' ) user_input_reset = 1;

	    if ( user_input_reset )
	    {
		    printf ( "Enter the polarity (0 for BIPOLAR, 1 for UNIPOLAR; default: 0): " );
	        fgets ( input_buffer, 20, stdin );
	        if ( input_buffer[0] == '\n' )
	        {
    		    dscadsettings.polarity = 0;
	        }
	        else
	        {
                sscanf ( input_buffer, "%d", &intBuff );
    		    dscadsettings.polarity = intBuff;
	        }

		    printf ( "Enter 0 for Single-Ended, 1 for Differential (default: 0): " );
	        fgets ( input_buffer, 20, stdin );
	        if ( input_buffer[0] == '\n' )
	        {
    		    dscadsettings.addiff = 0;
	        }
	        else
	        {
                sscanf ( input_buffer, "%d", &intBuff );
    		    dscadsettings.addiff = (BYTE) intBuff;
	        }

		    // The Helios only has one physical 10V range.
		    dscadsettings.range = RANGE_10;

		    printf ( "Enter the gain\n(0 for GAIN 1, 1 for GAIN 2, 2 for GAIN 4, 3 for GAIN 8; default: 0): " );
	        fgets ( input_buffer, 20, stdin );
	        if ( input_buffer[0] == '\n' )
	        {
    		    dscadsettings.gain = 0;
	        }
	        else
	        {
                sscanf ( input_buffer, "%d", &intBuff );
    		    dscadsettings.gain = (BYTE) intBuff;
	        }

		    // This feature will load the A/D calibration settings from the EEPROM

		    printf ( "Load corresponding A/D Calibration settings? (0: no,1: yes; default: 0): " );
	        fgets ( input_buffer, 20, stdin );
	        if ( input_buffer[0] == '\n' )
	        {
    		    dscadsettings.load_cal = 0;
	        }
	        else
	        {
                sscanf ( input_buffer, "%d", &intBuff );
       		    dscadsettings.load_cal = (BYTE)intBuff;
	        }

		    dscadsettings.current_channel = 0;

		    if ( ( result = dscADSetSettings ( dscb, &dscadsettings ) ) != DE_NONE )
		    {
			    dscGetLastError(&errorParams);
			    fprintf ( stderr, "dscADSetSettings error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );
			    return 0;
		    }

	        //=========================================================================
	        //
	        // IV. AD SCAN INITIALIZATION
	        //
	        //=========================================================================

		    printf ( "\nAD SCAN INITIALIZATION\n" );

            printf ( "(Single-Ended (0-%d)/Differential (0-%d))\n", HELIOS_MAX_AD_CHANNEL_NUMBER, HELIOS_MAX_AD_CHANNEL_NUMBER / 2 );	
		    printf ( "Enter the lowest channel in the scan range (0-%d; default: 0): ", HELIOS_MAX_AD_CHANNEL_NUMBER );
	        fgets ( input_buffer, 20, stdin );
	        if ( input_buffer[0] == '\n' )
	        {
		        dscadscan.low_channel = 0;
	        }
	        else
	        {
                sscanf ( input_buffer, "%d", &intBuff );
    		    dscadscan.low_channel = (BYTE) intBuff;
	        }

   		    printf ( "Enter the highest channel in the scan range (0-%d; default: %d): ", HELIOS_MAX_AD_CHANNEL_NUMBER, HELIOS_MAX_AD_CHANNEL_NUMBER );
	        fgets ( input_buffer, 20, stdin );
	        if ( input_buffer[0] == '\n' )
	        {
		        dscadscan.high_channel = HELIOS_MAX_AD_CHANNEL_NUMBER;
	        }
	        else
	        {
                sscanf ( input_buffer, "%d", &intBuff );
    		    dscadscan.high_channel = (BYTE) intBuff;
	        }

		    dscadscan.gain = dscadsettings.gain;
		    user_input_reset = 0;
	    }

		if ( ( result = dscADScan ( dscb, &dscadscan, samples ) ) != DE_NONE )
		{
			dscGetLastError(&errorParams);
			fprintf ( stderr, "dscADScan error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );
			return 0;
		}

		printf ("\nChannel  A/D Code    Voltage\n");

		for ( i = 0, k = dscadscan.low_channel; i < dscadscan.high_channel - dscadscan.low_channel + 1; i += 1, k += 1 )
		{
			if (dscadsettings.polarity)
				printf ("%2d\t%6.3hd\t",k, dscadscan.sample_values[i] + 32768);
			else
				printf ("%2d\t%6.3hd\t",k, dscadscan.sample_values[i] );
			if ( dscADCodeToVoltage(dscb, dscadsettings, dscadscan.sample_values[i], &voltage) != DE_NONE)
			{
				dscGetLastError(&errorParams);
				fprintf ( stderr, "dscADCodeToVoltage error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );
				free(samples);
				return 0;
			}

			printf ( "%6.3lfV\n", voltage );
		}

		printf ("Press 'q' for quit; ENTER to continue; 'i' to start over: ");

		fgets ( input_buffer, 20, stdin );
		temp_char = input_buffer[0];
	}
	while ( input_buffer[0] != 'q' );

	//=========================================================================
	// VI. CLEANUP
	//
	//	   Cleanup any remnants left by the program and free the resources used
	//	   by the driver.
	//
	//     STEPS TO FOLLOW:
	//
	//	   1. free the memory allocated for sample values
	//	   2. free the driver resources
	//=========================================================================

	free( samples );

	// Setting single-ended mode, so as to avoid confusion in other demos.
	dscadsettings.addiff = 0;

	if ( ( result = dscADSetSettings( dscb, &dscadsettings ) ) != DE_NONE )
	{
		dscGetLastError(&errorParams);
		fprintf ( stderr, "dscADSetSettings error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );
		return 0;
	}

	
	dscFree();

	printf ( "\nDSCADScan completed.\n" );

	return 0;
} // end main()


#ifdef _WIN32_WCE
int WINAPI WinMain(	HINSTANCE hInstance,
					HINSTANCE hPrevInstance,
					LPTSTR    lpCmdLine,
					int       nCmdShow)
{
	main();

	return 0;
}

#endif
