//=============================================================================
// (c) Copyright 2009 Diamond Systems Corporation. Use of this source code
// is subject to the terms of Diamond Systems' Software License Agreement.
// Diamond Systems provides no warranty of proper performance if this
// source code is modified.
//
// File: DSCADScanInt.c   v1.00
//
// Depedency :	Universal Driver version 6.02
//				DSCUD.H - Include file.
//				DSCUDBCL.LIB - Library file for MS-DOS
//						( USE BC 5.02 compiler )
//				LIBDSCUD_6.02_1.a	- Library file for Linux 2.6.23
//						( USE GCC 3.xx compiler )
//				DSCUD.LIB	- Static library file for Windows XP
//						( USE VC++ 6.0 or Visual Studio 2005 )
//				DSCUD_API.LIB	- Static library for Windows CE 6.0
//						( Use Visual Studio 2005 with Platform Builder 6.0 )
// Description: This sample code demonstrates the usage of UD function
//				dscADScanInt(). This sample code describes the usage
//				for Helios SBC. The driver will use the incumbent OS
//              in conjunction with the interrupt selected to transfer many
//              values from the A/D converter to user space, and once stopped,
//              will display a single set of those channel values,
//              interpretting them based on the mode given by the user.
//              Each A/D hardware scan consists of a sweep of all the
//              selected channels at a time, rather than only one channel.
// ************************* History ********************
// Version		Engineer		Date		Description
// 1.0			James Moore		07/27/09	Created.
//=============================================================================

#include <stdio.h>

#ifdef _WIN32
#ifndef _WIN32_WCE
#include <conio.h>
#endif

#include <windows.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>
// diamond driver includes
#include "dscud.h"
#endif

#ifdef _WIN32_WCE

#include <string.h>
#include <Winsock2.h>


static int kbhit()
{
	int i;
	int result=0;


	result |= GetAsyncKeyState(VK_RETURN);
	result |= GetAsyncKeyState(VK_SPACE);

	if (result != 0)
	{
		getchar();
		return 1;
	}

	//number keys, 0-9
	for (i=48; i<=57; i++)
		result |= GetAsyncKeyState(i);

	if (result != 0)
	{
		getchar();
		return 1;
	}

	//capital character keys, A-Z
	for (i=65; i<=90; i++)
		result |= GetAsyncKeyState(i);

	if (result != 0)
	{
		getchar();
		return 1;
	}

	//lower case keys, a-z
	for (i=97; i<=122; i++)
		result |= GetAsyncKeyState(i);

	if (result != 0)
	{
		getchar();
		return 1;
	}

	return result;
}

#endif


// DOS
#ifdef __BORLANDC__
#include <conio.h>
#include <dos.h>
#include <stdlib.h>
#include <math.h>
#include <mem.h>
// diamond driver includes
#include "dscud.h"
#endif

// Linux
#if defined(linux)
#include <stdlib.h>
#include <math.h>
#include <time.h>
#include <string.h>
// diamond driver includes
#include "dscud.h"

#include <sys/time.h>

static int kbhit()
{
	struct timeval timeout;
	fd_set rfds;

	timeout.tv_sec = 0;
	timeout.tv_usec = 0;

	FD_ZERO(&rfds);
	FD_SET(0, &rfds);

	if ( select(0+1, &rfds, NULL, NULL, &timeout) > 0 )
		return getchar();

	return 0;
}
#endif

// macros defined
#define SLEEP_TIME 1000

// var declarations
BYTE result; // returned error code
DSCB dscb;   // handle used to refer to the board
DSCCB dsccb; // structure containing board settings
DSCADSETTINGS dscadsettings; // structure containing A/D conversion settings
DSCAIOINT dscaioint; // structure containing I/O interrupt settings
DSCS dscs;       // structure containing interrupt-based sampling status information
DFLOAT voltage;  // actual voltage
ERRPARAMS errorParams;  // structure for returning error code and error string
int intBuff;            // temp variable of size int
long longBuff;          // temp variable of size long
float floatBuff;        // temp variable of size float
int i = 0;       // miscellaneous counter
int k = 0;       // channel print-out counter

#define HELIOS_DEFAULT_BASE_ADDRESS 0x280
#define HELIOS_MAX_AD_CHANNEL_NUMBER 15

char input_buffer[20]; // Buffer for parsing user character input.

int user_input_reset = 0;
char temp_char = 0;

//=============================================================================
// Name: main()
// Desc: Starting function that calls the driver functions used
//
//		 NOTE: By convention, you should capture the BYTE return value for each
//		 driver API call, and check the error code.
//
//     STEPS TO FOLLOW:
//
//	     I. Driver Initialization
//	    II. Board Initialization
//	   III. AD Settings Initialization
//		IV. I/O Interrupt Settings Initialization
//	     V. Sampling and Output
//	    VI. Cleanup
//
//=============================================================================

int main()
{
	//=========================================================================
	// I. DRIVER INITIALIZATION
	//
	//    Initializes the DSCUD library.
	//
	//    STEPS TO FOLLOW:
	//
	//	  1. initialize the driver, using the driver version for validation
	//=========================================================================

	if ( dscInit ( DSC_VERSION ) != DE_NONE )
	{
		dscGetLastError(&errorParams);
		fprintf ( stderr, "dscInit error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );
		return 0;
	}

	//=========================================================================
	// II. BOARD INITIALIZATION
	//
	//	   Initialize the HELIOS board. This function passes the various
	//	   hardware parameters to the driver and resets the hardware.
	//
	//     STEPS TO FOLLOW:
	//
	//	   1. set the board type to DSC_HELIOS for HELIOS board
	//	   2. set the base address to 0x280
	//	   3. set the interrupt level 5
	//	   4. intialize and register the board with the driver, after which
	//		  the struct, dscb, now holds the handle for the board
	//=========================================================================

	printf ( "\nHELIOS BOARD INITIALIZATION:\n" );

	printf ( "Enter the base address (default 0x280, hit ENTER) : " );
	fgets ( input_buffer, 20, stdin );
	if ( input_buffer[0] == '\n' )
	{
    	dsccb.base_address = HELIOS_DEFAULT_BASE_ADDRESS;
   	}
	else
	{
	    sscanf ( input_buffer, "%hx", &dsccb.base_address );
	}
	dsccb.int_level = 5;

	if ( dscInitBoard ( DSC_HELIOS, &dsccb, &dscb ) != DE_NONE )
	{
		dscGetLastError(&errorParams);
		fprintf ( stderr, "dscInitBoard error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );
		return 0;
	}

	//=========================================================================
	// III. AD SETTINGS INITIALIZATION
	//
	//	    Initialize the structure containing the AD conversion settings and
	//		then pass it to the driver.
	//
	//      STEPS TO FOLLOW:
	//
	//		1. set the polarity (must be BIPOLAR or UNIPOLAR)
	//		2. set the gain (must be GAIN_1, GAIN_2, GAIN_4, or GAIN_8)
	//		3. set the current channel to be sampled (must be between 0-15)
	//		4. initialize the AD conversion with these settings
	//=========================================================================

	printf ( "\nAD SETTINGS INITIALIZATION\n" );

	memset(&dscadsettings, 0, sizeof(DSCADSETTINGS));

	dscadsettings.range = RANGE_10;

    printf ( "Enter the polarity (0 for BIPOLAR, 1 for UNIPOLAR; default: 0): " );
	fgets ( input_buffer, 20, stdin );
	if ( input_buffer[0] == '\n' )
	{
    	dscadsettings.polarity = 0;
	}
	else
	{
        sscanf ( input_buffer, "%d", &intBuff );
    	dscadsettings.polarity = (BYTE) intBuff;
	}

	printf ( "Enter 0 for Single-Ended, 1 for Differential (default: 0): " );
	fgets ( input_buffer, 20, stdin );
	if ( input_buffer[0] == '\n' )
	{
    	dscadsettings.addiff = 0;
	}
	else
	{
        sscanf ( input_buffer, "%d", &intBuff );
    	dscadsettings.addiff = (BYTE) intBuff;
	}

	// Helios only has one physical 10V range.
	dscadsettings.range = RANGE_10;

	printf ( "Enter the gain\n(0 for GAIN 1, 1 for GAIN 2, 2 for GAIN 4, 3 for GAIN 8; default: 0): " );
	fgets ( input_buffer, 20, stdin );
	if ( input_buffer[0] == '\n' )
	{
    	dscadsettings.gain = 0;
	}
	else
	{
        sscanf ( input_buffer, "%d", &intBuff );
    	dscadsettings.gain = (BYTE) intBuff;
	}

	// This feature will load the A/D calibration settings from the EEPROM

	printf ( "Load corresponding A/D Calibration settings? (0: no,1: yes; default: 0): " );
	fgets ( input_buffer, 20, stdin );
	if ( input_buffer[0] == '\n' )
	{
    	dscadsettings.load_cal = 0;
	}
	else
	{
        sscanf ( input_buffer, "%d", &intBuff );
    	dscadsettings.load_cal = (BYTE)intBuff;
	}

	dscadsettings.current_channel = 0;

	if ( ( result = dscADSetSettings ( dscb, &dscadsettings ) ) != DE_NONE )
	{
		dscGetLastError(&errorParams);
		fprintf ( stderr, "dscADSetSettings error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );
		return 0;
	}

	//=========================================================================
	// IV. I/O INTERRUPT SETTINGS INITIALIZATION
	//
	//	   Initialize the structure containing the analog I/O interrupt
	//	   settings.
	//
	//	   NOTE: You must allocate space for the buffer holding the returned
	//		     sample values. Also, be generous in allocating storage.
	//			 Allocating insufficient memory to hold sample data will result
	//			 in improper behavior of the driver, such as hanging interrupt
	//			 operations or assertion errors.
	//
	//     STEPS TO FOLLOW:
	//
	//	   1.  set the number of conversions (must be a multiple of the fifo
	//		   depth)
	//	   2.  set the conversion rate (must be less than 100 kHz)
	//	   3.  set the cycle flag
	//	   4.  set the internal clock flag
	//	   5.  set the low channel (must be between 0-15)
	//	   6.  set the high channel (must be between 0-15)
	//	   7.  set the external gate enable flag
	//	   8.  set the internal clock gate flag
	//	   9.  set the fifo enable flag
	//	   10. set the fifo depth (must be 46 or less )
	//	   11. allocate memory for the sample values
	//=========================================================================

	printf ( "\nI/O INTERRUPT SETTINGS INITIALIZATION\n" );

	memset(&dscaioint, 0, sizeof(DSCAIOINT));

	printf ( "Enter the number of conversions\n(must be a multiple of FIFO depth; default: 1024) : " );
	fgets ( input_buffer, 20, stdin );
	if ( input_buffer[0] == '\n' )
	{
    	dscaioint.num_conversions = 1024;
	}
	else
	{
        sscanf ( input_buffer, "%ld", &longBuff );
    	dscaioint.num_conversions = (DWORD) longBuff;
	}

	printf ( "Enter the conversion rate in Hz (default: 100000) : " );
	fgets ( input_buffer, 20, stdin );
	if ( input_buffer[0] == '\n' )
	{
    	dscaioint.conversion_rate = 100000;
	}
	else
	{
        sscanf ( input_buffer, "%f", &floatBuff );
    	dscaioint.conversion_rate = (FLOAT) floatBuff;
	}

	printf ( "Enter the cycle flag (0 for FALSE, 1 for TRUE; default: 1) : " );
	fgets ( input_buffer, 20, stdin );
	if ( input_buffer[0] == '\n' )
	{
    	dscaioint.cycle = 1;
	}
	else
	{
        sscanf ( input_buffer, "%s", &intBuff );
    	dscaioint.cycle = (BOOL) intBuff;
	}

	printf ( "Enter the internal clock flag (0 for FALSE, 1 for TRUE; default: 1) : " );
	fgets ( input_buffer, 20, stdin );
	if ( input_buffer[0] == '\n' )
	{
    	dscaioint.internal_clock  = 1;
	}
	else
	{
        sscanf ( input_buffer, "%d", &intBuff );
    	dscaioint.internal_clock  = (BOOL) intBuff;
	}

	printf ( "Enter the lowest channel in the scan range (0-%d; default: 0) : ", HELIOS_MAX_AD_CHANNEL_NUMBER );
	fgets ( input_buffer, 20, stdin );
	if ( input_buffer[0] == '\n' )
	{
    	dscaioint.low_channel = 0;
	}
	else
	{
        sscanf ( input_buffer, "%d", &intBuff );
    	dscaioint.low_channel = (BYTE) intBuff;
	}

	printf ( "Enter the highest channel in the scan range (0-%d; default: %d) : ", HELIOS_MAX_AD_CHANNEL_NUMBER, HELIOS_MAX_AD_CHANNEL_NUMBER );
	fgets ( input_buffer, 20, stdin );
	if ( input_buffer[0] == '\n' )
	{
        dscaioint.high_channel = HELIOS_MAX_AD_CHANNEL_NUMBER;
	}
	else
	{
        sscanf ( input_buffer, "%d", &intBuff );
        dscaioint.high_channel = (BYTE) intBuff;
	}

	dscaioint.external_gate_enable = 0; // can enable it if need be

	dscaioint.internal_clock_gate = 0;   // can enable it if need be

	dscaioint.fifo_enab = 1;


    // In standard mode, the FIFO depth parameter here, the amount of
    //   samples that the FIFO will store before generating an
    //   interrupt, 0-48 can be selected. Enhanced mode will be
    //   enabled for larger numbers. If the number is 1024 or 2048,
    //   those numbers will be used. Otherwise, a number ranging from (49) to
    //   (255) will cause the DSCUD and FPGA to-- if the number is valid for
    //   the rest of the variables, such as number of samples to take--
    //   multiply it by 8 (392-2040). That number will be the threshold at which
    //   the FPGA will trigger an interrupt.
    //
	printf ( "Enter the FIFO usage depth\n(0-48 standard mode, 49-255 extended mode; default: 1024): " );
	fgets ( input_buffer, 20, stdin );
	if ( input_buffer[0] == '\n' )
	{
    	dscaioint.fifo_depth = 1024;
	}
	else
	{
        sscanf ( input_buffer, "%d", &intBuff );
    	dscaioint.fifo_depth = (WORD) intBuff;
	}

	// Dump Threshold here is the threshold at which the data from
	//   kernel space is copied to the structure submitted in user space.

	printf ( "Enter the transfer to user memory threshold\n( must be less or equal to num conversions; default: 1024 ) : " );
    fgets ( input_buffer, 20, stdin );
	if ( input_buffer[0] == '\n' )
	{
    	dscaioint.dump_threshold = 1024;
	}
	else
	{
        sscanf ( input_buffer, "%ld", &longBuff );
    	dscaioint.dump_threshold = (DWORD)longBuff;
	}

	dscaioint.sample_values = (DSCSAMPLE*)malloc( sizeof(DSCSAMPLE) * dscaioint.num_conversions );


	//=========================================================================
	// V. SCANNING AND OUTPUT
	//
	//    Perform the actual sampling and then output the results. To calculate
	//	  the actual input voltages, we must convert the sample code (which
	//	  must be cast to a short to get the correct code) and then plug it
	//	  into one of the formulas located in the manual for your board (under
	//	  "A/D Conversion Formulas"). For example, if you are using a bipolar
	//	  input range and 32-bit signed integers:
	//
	//	  Input voltage = (AD Code / 32768) x Full-Scale Voltage
	//
	//	  For interrupt-based scanning, we perform scan several channels per
	//	  interrupt.
	//
	//    STEPS TO FOLLOW:
	//
	//	  1. reset the number of transfers to 0
	//	  2. set the operation type to interrupt-based
	//    3. get the samples from the board
	//	  4. wait for the board to signal that the operation has finished
	//    5. convert the results to a signed short to obtain a readout from
	//       -32768 to +32767 which scales to a range from -10V to +10V and
	//	     output the sample codes and the actual voltages to console
	//    6. calculate the input voltages by converting the sample code and
	//		 output the results
	//	  7. repeat steps 1-6 until a key is pressed
	//=========================================================================

	printf ( "\nSAMPLING AND OUTPUT\n" );

	do
	{


	if ( temp_char == 'i' ) user_input_reset = 1;

	if ( user_input_reset )
	{

        printf ( "Enter the polarity (0 for BIPOLAR, 1 for UNIPOLAR; default: 0): " );
	    fgets ( input_buffer, 20, stdin );
	    if ( input_buffer[0] == '\n' )
	    {
    	    dscadsettings.polarity = 0;
	    }
	    else
	    {
            sscanf ( input_buffer, "%d", &intBuff );
    	    dscadsettings.polarity = (BYTE) intBuff;
	    }

	    printf ( "Enter 0 for Single-Ended, 1 for Differential (default: 0): " );
	    fgets ( input_buffer, 20, stdin );
	    if ( input_buffer[0] == '\n' )
	    {
    	    dscadsettings.addiff = 0;
	    }
	    else
	    {
            sscanf ( input_buffer, "%d", &intBuff );
    	    dscadsettings.addiff = (BYTE) intBuff;
	    }

	    // Helios only has one physical 10V range.
	    dscadsettings.range = RANGE_10;

	    printf ( "Enter the gain\n(0 for GAIN 1, 1 for GAIN 2, 2 for GAIN 4, 3 for GAIN 8; default: 0): " );
	    fgets ( input_buffer, 20, stdin );
	    if ( input_buffer[0] == '\n' )
	    {
    	    dscadsettings.gain = 0;
	    }
	    else
	    {
            sscanf ( input_buffer, "%d", &intBuff );
    	    dscadsettings.gain = (BYTE) intBuff;
	    }

	    // This feature will load the A/D calibration settings from the EEPROM

	    printf ( "Load corresponding A/D Calibration settings? (0: no,1: yes; default: 0): " );
	    fgets ( input_buffer, 20, stdin );
	    if ( input_buffer[0] == '\n' )
	    {
    	    dscadsettings.load_cal = 0;
	    }
	    else
	    {
            sscanf ( input_buffer, "%d", &intBuff );
    	    dscadsettings.load_cal = (BYTE)intBuff;
	    }

		dscadsettings.current_channel = 0;

		if ( ( result = dscADSetSettings( dscb, &dscadsettings ) ) != DE_NONE )
		{
			dscGetLastError(&errorParams);
			fprintf ( stderr, "dscADSetSettings error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );
			return 0;
		}

	    //=========================================================================
	    // IV. I/O INTERRUPT SETTINGS INITIALIZATION
	    //=========================================================================

	    printf ( "\nI/O INTERRUPT SETTINGS INITIALIZATION\n" );

	    printf ( "Enter the number of conversions\n(must be a multiple of FIFO depth; default: 1024) : " );
	    fgets ( input_buffer, 20, stdin );
	    if ( input_buffer[0] == '\n' )
	    {
    	    dscaioint.num_conversions = 1024;
	    }
	    else
	    {
            sscanf ( input_buffer, "%ld", &longBuff );
    	    dscaioint.num_conversions = (DWORD) longBuff;
	    }
	    printf ( "Enter the conversion rate in Hz (default: 100000) : " );
	    fgets ( input_buffer, 20, stdin );
	    if ( input_buffer[0] == '\n' )
	    {
    	    dscaioint.conversion_rate = 100000;
	    }
	    else
	    {
            sscanf ( input_buffer, "%f", &floatBuff );
    	    dscaioint.conversion_rate = (FLOAT) floatBuff;
	    }

	    printf ( "Enter the cycle flag (0 for FALSE, 1 for TRUE; default: 1) : " );
	    fgets ( input_buffer, 20, stdin );
	    if ( input_buffer[0] == '\n' )
	    {
    	    dscaioint.cycle = 1;
	    }
	    else
	    {
            sscanf ( input_buffer, "%s", &intBuff );
    	    dscaioint.cycle = (BOOL) intBuff;
	    }

	    printf ( "Enter the internal clock flag (0 for FALSE, 1 for TRUE; default: 1) : " );
	    fgets ( input_buffer, 20, stdin );
	    if ( input_buffer[0] == '\n' )
	    {
    	    dscaioint.internal_clock  = 1;
	    }
	    else
	    {
            sscanf ( input_buffer, "%d", &intBuff );
    	    dscaioint.internal_clock  = (BOOL) intBuff;
	    }

	    printf ( "Enter the lowest channel in the scan range (0-%d; default: 0) : ", HELIOS_MAX_AD_CHANNEL_NUMBER );
	    fgets ( input_buffer, 20, stdin );
	    if ( input_buffer[0] == '\n' )
	    {
    	    dscaioint.low_channel = 0;
	    }
	    else
	    {
            sscanf ( input_buffer, "%d", &intBuff );
    	    dscaioint.low_channel = (BYTE) intBuff;
	    }

	    printf ( "Enter the highest channel in the scan range (0-%d; default: %d) : ", HELIOS_MAX_AD_CHANNEL_NUMBER, HELIOS_MAX_AD_CHANNEL_NUMBER );
	    fgets ( input_buffer, 20, stdin );
	    if ( input_buffer[0] == '\n' )
	    {
            dscaioint.high_channel = HELIOS_MAX_AD_CHANNEL_NUMBER;
	    }
	    else
	    {
            sscanf ( input_buffer, "%d", &intBuff );
            dscaioint.high_channel = (BYTE) intBuff;
	    }

	    dscaioint.external_gate_enable = 0; // can enable it if need be

	    dscaioint.internal_clock_gate = 0;   // can enable it if need be

	    dscaioint.fifo_enab = 1;

	    printf ( "Enter the FIFO usage depth\n(0-48 standard mode, 49-255 extended mode; default: 1024): " );
	    fgets ( input_buffer, 20, stdin );
	    if ( input_buffer[0] == '\n' )
	    {
    	    dscaioint.fifo_depth = 1024;
	    }
	    else
	    {
            sscanf ( input_buffer, "%d", &intBuff );
    	    dscaioint.fifo_depth = (WORD) intBuff;
	    }

	    // Dump Threshold here is the threshold at which the data from
	    //   kernel space is copied to the structure submitted in user space.

	    printf ( "Enter the transfer to user memory threshold\n(must be less or equal to num conversions; default: 1024 ) : " );
        fgets ( input_buffer, 20, stdin );
	    if ( input_buffer[0] == '\n' )
	    {
    	    dscaioint.dump_threshold = 1024;
	    }
	    else
	    {
            sscanf ( input_buffer, "%ld", &longBuff );
    	    dscaioint.dump_threshold = (DWORD)longBuff;
	    }

		// Preserves "Hit enter to repeat" behaviour
		user_input_reset = 0;

    } // End user input reset


		if ( ( result = dscADScanInt ( dscb, &dscaioint ) ) != DE_NONE )
		{
			dscGetLastError(&errorParams);
			fprintf ( stderr, "dscADScanInt error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );
			free( dscaioint.sample_values ); // remember to deallocate malloc() memory
			return 0;
		}

		// check interrupt status
		dscs.transfers = 0;
		dscs.overflows = 0;
		dscs.op_type = OP_TYPE_INT;

		do
        {
			dscSleep(SLEEP_TIME);
			dscGetStatus(dscb, &dscs);
			printf ( "Fifo Overflows %lu, A/D Int Trans in progress: %lu  Total Trans %lu\n", dscs.overflows, dscs.transfers, dscs.total_transfers);
        } while ( dscs.op_type != OP_TYPE_NONE && !kbhit() );

		// cancel interrupts manually for recycled mode or if interrupts are still running
		if ( dscs.op_type != OP_TYPE_NONE )
		{
			if ( ( result = dscCancelOp ( dscb ) ) != DE_NONE )
			{
				dscGetLastError(&errorParams);
				fprintf ( stderr, "dscCancelOp error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );
				free( dscaioint.sample_values ); // remember to deallocate malloc() memory
				return 0;
			}
		}


       	printf ("\nChannel  A/D Code    Voltage\n");

		for ( i = 0, k = dscaioint.low_channel; i < dscaioint.high_channel - dscaioint.low_channel + 1; i += 1, k += 1 )
		{
			if ( dscadsettings.polarity )
				printf ("%2d\t%6.3hd\t",k, dscaioint.sample_values[i] + 32768);
			else
				printf ("%2d\t%6.3hd\t",k, dscaioint.sample_values[i] );
			if ( dscADCodeToVoltage ( dscb, dscadsettings, dscaioint.sample_values[i], &voltage ) != DE_NONE )
			{
				dscGetLastError(&errorParams);
				fprintf ( stderr, "dscADCodeToVoltage error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );
				free(dscaioint.sample_values);
				return 0;
			}
			printf ( "%6.3lfV\n", voltage );
		}

		printf ( "\nPress 'q' for quit; ENTER to continue; 'i' to start over: " );

		fgets ( input_buffer, 20, stdin );
		temp_char = input_buffer[0];
	}
	while ( input_buffer[0] != 'q' );

	//=========================================================================
	// VI. CLEANUP
	//
	//	   Cleanup any remnants left by the program and free the resources used
	//	   by the driver.
	//
	//     STEPS TO FOLLOW:
	//
	//	   1. free the memory allocated for sample values
	//	   2. free the driver resources
	//=========================================================================

	free( dscaioint.sample_values );

	// Setting single-ended mode, so as to avoid confusion in other demos.
	dscadsettings.addiff = 0;

	if ( ( result = dscADSetSettings( dscb, &dscadsettings ) ) != DE_NONE )
	{
		dscGetLastError(&errorParams);
		fprintf ( stderr, "dscADSetSettings error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );
		return 0;
	}

	dscFree();

	printf ( "\nDSCADScanInt completed. \n" );

	return 0;
} // end main()


#ifdef _WIN32_WCE
int WINAPI WinMain(	HINSTANCE hInstance,
					HINSTANCE hPrevInstance,
					LPTSTR    lpCmdLine,
					int       nCmdShow)
{
	main();

	return 0;
}

#endif
