//=============================================================================
// (c) Copyright 2009 Diamond Systems Corporation. Use of this source code
// is subject to the terms of Diamond Systems' Software License Agreement.
// Diamond Systems provides no warranty of proper performance if this
// source code is modified.
//
// File: DSCDAAutoCal.c   v1.00
//
// Depedency :	Universal Driver version 6.02
//				DSCUD.H - Include file.
//				DSCUDBCL.LIB - Library file for MS-DOS
//						( USE BC 5.02 compiler )
//				LIBDSCUD_6.02_1.a	- Library file for Linux 2.6.23
//						( USE GCC 3.xx compiler )
//				DSCUD.LIB	- Static library file for Windows XP
//						( USE VC++ 6.0 or Visual Studio 2005 )
//				DSCUD_API.LIB	- Static library for Windows CE 6.0
//						( Use Visual Studio 2005 with Platform Builder 6.0 )
// Description: This sample code demonstrates the usage of UD function
//				dscDAAutoCal(). This sample code describes the usage
//				for Helios SBC. AutoCalibration will use the onboard
//              hardware to calibrate the D/A circuitry on the board,
//              then verify that the process results fall within tolerances.
//              The calibration values will be stored in the EEPROM,
//              along with one set of values to be loaded at boot time.
// ************************* History ********************
// Version		Engineer		Date		Description
// 1.0			James Moore		07/27/09	Created.
//=============================================================================

#include <stdio.h>

#ifdef _WIN32
#ifndef _WIN32_WCE
#include <conio.h>
#endif

#include <windows.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>
// diamond driver includes
#include "dscud.h"
#endif


#ifdef _WIN32_WCE

#include <string.h>
#include <Winsock2.h>

#endif


// DOS
#ifdef __BORLANDC__
#include <conio.h>
#include <dos.h>
#include <stdlib.h>
#include <math.h>
// diamond driver includes
#include "dscud.h"
#endif

// Linux
#if defined(linux)
#include <stdlib.h>
#include <math.h>
#include <time.h>
// diamond driver includes
#include "dscud.h"

#include <sys/time.h>

#endif

#define OFFSET_LSB_TOLERANCE 2
#define GAIN_LSB_TOLERANCE 2

// var declarations
BYTE result; // returned error code
DSCB board;  // handle used to refer to the board
DSCCB dsccb; // structure containing board settings
DSCADSETTINGS dscadsettings; // structure containing A/D conversion settings
DSCDACALPARAMS dacalparams; // structure containing auto-calibration settings
ERRPARAMS errorParams; // structure for returning error code and error string
int i;	     // miscellaneous counter
int intBuff; // temporary variable with size int

#define HELIOS_DEFAULT_BASE_ADDRESS 0x280

char input_buffer[20]; // Buffer for parsing user character input.

//=============================================================================
// Name: main()
// Desc: Starting function that calls the driver functions used
//
//		 NOTE: By convention, you should capture the BYTE return value for each
//		 driver API call, and check the error code.
//
//	     I. Driver Initialization
//	    II. Board Initialization
//	   III. AutoCal Settings Initialization
//	    IV. Auto-Calibration
//		 V. Calibration Verification
//	    VI. Cleanup
//
//=============================================================================

int main( void )
{
	//=========================================================================
	// I. DRIVER INITIALIZATION
	//
	//    Initializes the DSCUD library.
	//
	//=========================================================================

	if( dscInit ( DSC_VERSION ) != DE_NONE )
	{
		dscGetLastError(&errorParams);
		fprintf ( stderr, "dscInit error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );
		return 0;
	}

	//=========================================================================
	// II. BOARD INITIALIZATION
	//
	//	   Initialize the Helios board. This function passes the various
	//	   hardware parameters to the driver and resets the hardware.
	//
	//=========================================================================

	printf ( "\nHELIOS BOARD INITIALIZATION:\n" );

	printf ( "Enter the base address (default 0x280, hit ENTER) : " );
	fgets ( input_buffer, 20, stdin );
	if ( input_buffer[0] == '\n' )
	{
		dsccb.base_address = HELIOS_DEFAULT_BASE_ADDRESS;
	}
	else
	{
		sscanf ( input_buffer, "%hx", &dsccb.base_address );
	}

	dsccb.int_level = 5;

	if ( dscInitBoard ( DSC_HELIOS, &dsccb, &board ) != DE_NONE )
	{
		dscGetLastError(&errorParams);
		fprintf ( stderr, "dscInitBoard error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );
		return 0;
	}


	//=========================================================================
	// III. AUTOCAL SETTINGS INITIALIZATION
	//
	//		Initialize the structure containing the DA autocal settings.
	//		Set dacalparams.darange_calibrate for the MODE to calibrate. Set boot range
	//		for the mode the board to be in upon power up.
	//
	//=========================================================================

	do {
		printf ( "\n\nAUTOCAL SETTINGS INITIALIZATION:\n" );

        printf ( "\nRange ; meaning\n0:\t5V\tunipolar\n1:\t10V\tunipolar\n2:\tNot used\n3:\tNot used\n" );
        printf ( "4:\t2.5V\tbipolar\n5:\t5V\tbipolar\n6:\t10V\tbipolar\n7:\tNot used\n" );

		printf ( "Enter the range to calibrate (0-7)\nor 255 to calibrate all ranges (default: 255): " );
	    fgets ( input_buffer, 20, stdin );
	    if ( input_buffer[0] == '\n' )
	    {
    		dacalparams.darange_calibrate = 255;
	    }
	    else
	    {
            sscanf ( input_buffer, "%d", &intBuff );
    		dacalparams.darange_calibrate = (BYTE) intBuff;
	    }

		// Since the Helios A/D and D/A voltage ranges are software programmable, this setting
		//   will only load the EEPROM calibration settings, not the mode settings. The user
		//   must change the mode settings (such as 5V unipolar or 10V bipolar). See manual for more details.

		printf ( "Enter the DA calibration settings to load at boot time (0-7; default: 0): " );
	    fgets ( input_buffer, 20, stdin );
	    if ( input_buffer[0] == '\n' )
	    {
    		dacalparams.boot_darange = 0;
	    }
	    else
	    {
            sscanf ( input_buffer, "%d", &intBuff );
    		dacalparams.boot_darange = (BYTE) intBuff;
	    }

	//=========================================================================
	// IV. AUTO-CALIBRATION
	//
	//	   Perform the actual DA auto-calibration of the board.
	//
	//=========================================================================

		printf ( "\nAUTO-CALIBRATION:\n" );
		printf ( "Starting DA auto-calibration...this will take a few seconds\n" );

		if( dscDAAutoCal ( board, &dacalparams ) != DE_NONE )
		{
			dscGetLastError(&errorParams);
			fprintf ( stderr, "dscADAutoCal error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );
			return 0;
		}

		printf ( "Completed\n\n" );

	//=========================================================================
	// V. CALIBRATION VERIFICATION
	//
	//	  To verify that the auto-calibration was successfully completed, you
	//	  should run dscADCalVerify() on the 15 possible configuration modes to
	//	  ensure that the gain and offset errors meet a specified tolerance (2
	//	  in this example). You can obtain the possible combinations by writing
	//	  a for loop and bit-wise operations similar to those used below.
	//
	//=========================================================================

		printf ( "\nCALIBRATION VERIFICATION:\n" );

        if ( dacalparams.darange_calibrate == 255 )
        {
		    for( i = 0; i < 8; i++ )
		    {
			    if ( i == 2 || i == 3 || i == 7 )
			    {
				    continue;
			    }

			    dacalparams.darange_calibrate = i;
			    if( ( result = dscDACalVerify( board, &dacalparams) ) != DE_NONE )
			    {
				    dscGetLastError(&errorParams);
				    fprintf ( stderr, "dscDACalVerify error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );
				    return 0;
			    }

			    printf ( "Configuration Mode: %d, Offset Error: %9.3f, Gain Error: %9.3f\n", i, dacalparams.offset, dacalparams.gain );

			    if ( fabs( dacalparams.offset ) > OFFSET_LSB_TOLERANCE ||
				     fabs( dacalparams.gain ) > GAIN_LSB_TOLERANCE )
				    printf ( "Value for offset or gain exceeded specified tolerance\n" );
			    else printf ( "Values for offset and gain met specified tolerance\n" );
		    }
        }
        else
        {
			if( ( result = dscDACalVerify( board, &dacalparams) ) != DE_NONE )
			{
				dscGetLastError(&errorParams);
				fprintf ( stderr, "dscDACalVerify error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );
				return 0;
			}

			printf ( "Configuration Mode: %d, Offset Error: %9.3f, Gain Error: %9.3f\n", dacalparams.darange_calibrate , dacalparams.offset, dacalparams.gain );

			if ( fabs( dacalparams.offset ) > OFFSET_LSB_TOLERANCE ||
				 fabs( dacalparams.gain ) > GAIN_LSB_TOLERANCE )
				printf ( "Value for offset or gain exceeded specified tolerance\n" );
			else printf ( "Values for offset and gain met specified tolerance\n" );

        }

        printf ( "\nPress 'q' to exit or ENTER to continue : " );

	    fgets ( input_buffer, 20, stdin );


    } while ( input_buffer[0] != 'q' );


	//=========================================================================
	// VI. CLEANUP
	//
	//	   Cleanup any remnants left by the program and free the resources used
	//	   by the driver.
	//
	//     STEPS TO FOLLOW:
	//
	//	   1. free the driver resources
	//=========================================================================

	dscFree();

	printf ( "\nDSCDAAutoCal completed. \n" );
	return 0;
} // end main()



#ifdef _WIN32_WCE
int WINAPI WinMain(	HINSTANCE hInstance,
					HINSTANCE hPrevInstance,
					LPTSTR    lpCmdLine,
					int       nCmdShow)
{
	main();

	return 0;
}

#endif
