//=============================================================================
// (c) Copyright 2009 Diamond Systems Corporation. Use of this source code
// is subject to the terms of Diamond Systems' Software License Agreement.
// Diamond Systems provides no warranty of proper performance if this
// source code is modified.
//
// File: Watchdog.c   v1.00
//
// Depedency :	Universal Driver version 6.02
//				DSCUD.H - Include file.
//				DSCUDBCL.LIB - Library file for MS-DOS 
//						( USE BC 5.02 compiler )
//				LIBDSCUD_6.02_1.a	- Library file for Linux 2.6.23
//						( USE GCC 3.xx compiler )
//				DSCUD.LIB	- Static library file for Windows XP
//						( USE VC++ 6.0 or Visual Studio 2005 )
//				DSCUD_API.LIB	- Static library for Windows CE 6.0
//						( Use Visual Studio 2005 with Platform Builder 6.0 )
// Description: This sample code demonstrates the usage of UD functions  
//				dscWatchdogTrigger(), dscWatchdogTrigger(), dscWatchdogDisable().
//              This sample code describes the usage for Helios SBC. The
//              current configuration of the code demonstrates how to reset
//              the board after the counter finishes.
// ************************* History ********************
// Version		Engineer		Date		Description		
// 1.0			Bhavin Shah		07/30/09	Created.
//=============================================================================

#include <stdio.h>

#ifdef _WIN32
#ifndef _WIN32_WCE
#include <conio.h>
#endif

#include <windows.h>
#include <stdlib.h>
#include <math.h>
// diamond driver includes
#include "dscud.h"
#endif

// DOS
#ifdef __BORLANDC__
#include <conio.h>
#include <dos.h>
#include <stdlib.h>
#include <math.h>
// diamond driver includes
#include "dscud.h"
#endif

// Linux
#if defined(linux)
#include <stdlib.h>
#include <math.h>
#include <time.h>
// diamond driver includes
#include "dscud.h"

#include <sys/time.h>
#endif

#define HELIOS_DEFAULT_BASE_ADDRESS 0x280

// var declarations
BYTE result; // returned error code
DSCB dscb;   // handle used to refer to the board
DSCCB dsccb; // structure containing board settings
ERRPARAMS errorParams;  // structure for returning error code and error string
float floatBuff;        // temp variable of size float
BYTE RebootTimer;  // timer program value
int i ;

char input_buffer[20]; // Holds the user input, one line at a time.

//=============================================================================
// Name: main()
// Desc: Starting function that calls the driver functions used
//
//		 NOTE: By convention, you should capture the BYTE return value for each
//		 driver API call, and check the error code.
//
//     STEPS TO FOLLOW:
//
//	     I. Driver Initialization
//	    II. Board Initialization
//	   III. Watchdog Setup & Run
//	    IV. Cleanup
//
//=============================================================================


int main()
{
	//=========================================================================
	// I. DRIVER INITIALIZATION
	//
	//    Initializes the DSCUD library.
	//
	//    STEPS TO FOLLOW:
	//
	//	  1. initialize the driver, using the driver version for validation
	//=========================================================================

	if ( dscInit ( DSC_VERSION ) != DE_NONE )
	{
		dscGetLastError(&errorParams);
		fprintf( stderr, "dscInit error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );
		return 0;
	}

	//=========================================================================
	// II. BOARD INITIALIZATION
	//
	//	   Initialize the HELIOS board. This function passes the various
	//	   hardware parameters to the driver and resets the hardware.
	//
	//     STEPS TO FOLLOW:
	//
	//	   1. set the board type to DSC_HELIOS for HELIOS board
	//	   2. set the base address to 0x280
	//	   3. set the interrupt level 5
	//	   4. intialize and register the board with the driver, after which
	//		  the struct, dscb, now holds the handle for the board
	//=========================================================================

	printf( "\nHELIOS BOARD INITIALIZATION:\n" );

	printf ( "Enter the base address (default 0x280, hit ENTER) : " );
	fgets ( input_buffer, 20, stdin );
	if ( input_buffer[0] == '\n' )
	{
		dsccb.base_address = HELIOS_DEFAULT_BASE_ADDRESS;
	}
	else
	{
		sscanf ( input_buffer, "%hx", &dsccb.base_address );
	}

    dsccb.int_level = 5;

	if ( dscInitBoard ( DSC_HELIOS, &dsccb, &dscb ) != DE_NONE )
	{
		dscGetLastError(&errorParams);
		fprintf( stderr, "dscInitBoard error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );
		return 0;
	}

	//=========================================================================
	// III. Watchdog Setup & Run
	//
	//	    Setup and enable the HELIOS Watchdog timer. Once the timer
	//		hits the threshold it will shutdown the HELIOS
	//
	//	    STEPS TO FOLLOW:
	//
	//	    1. Setup watchdog 1 with the required time.
	//			NOTE: This timer can achieve a max of 512 seconds delay.
	//	    2. Set the watchdog timer options
	//			NOTE: The parameters are defined in the Helios User's manual
	//			      This example sets the parameter as 0xD0 which corresponds
	//				  to Resetting the system upon watchdog timer expiration.
	//	    Inside an infinite loop trigger watchdog. If this function call is
	//		commented, the watchdog timer will restart the system after 4 seconds.
	//=========================================================================

	dscWatchdogEnable ( dscb , 0 , 5 , 0xD0 ) ; // Action = Reset board

	i = 0 ;
	while ( 1 )
	{
		// other application specific actions
		dscWatchdogTrigger ( dscb ) ;
		dscSleep (1000);
		printf ( "Watchdog trigger count ... %d\n" , i ++);
		if ( i == 10 )
		{
			printf ( "Not triggering watchdog NOW!!!\n");
			break;
		}
	}
	printf ("The system will now Reset after the configured time...");
	while (1);



	//=========================================================================
	// IV. CLEANUP
	//
	//	   Cleanup any remnants left by the program and free the resources used
	//	   by the driver.
	//
	//     STEPS TO FOLLOW:
	//
	//	   1. free the driver resources
	//=========================================================================

	dscFree();

	printf( "\nWatchdog completed.\n" );

	return 0;
} // end main()


#ifdef _WIN32_WCE
int WINAPI WinMain(	HINSTANCE hInstance,
					HINSTANCE hPrevInstance,
					LPTSTR    lpCmdLine,
					int       nCmdShow)
{
	main();

	return 0;
}

#endif
